@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright (c) 2013 OpenStack Foundation.
 # All rights reserved.
 #
@@ -13,27 +15,26 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import abc
+from abc import abstractmethod
 
-from neutron_lib.api import extensions as api_extensions
-from neutron_lib.api import faults
-from neutron_lib import constants
-from neutron_lib import exceptions
-from neutron_lib.exceptions import agent as agent_exc
-from neutron_lib.plugins import constants as plugin_constants
-from neutron_lib.plugins import directory
-from neutron_lib import rpc as n_rpc
-from oslo_log import log as logging
 import webob.exc
 
-from neutron._i18n import _
 from neutron.api import extensions
+from neutron.api.v2 import base
 from neutron.api.v2 import resource
-from neutron.api import wsgi
+from neutron.common import constants
+from neutron.common import exceptions
+from neutron.extensions import agent
+from neutron import manager
+from neutron.openstack.common import log as logging
+from neutron.plugins.common import constants as service_constants
 from neutron import policy
+from neutron import wsgi
+
 
 LOG = logging.getLogger(__name__)
 
+
 L3_ROUTER = 'l3-router'
 L3_ROUTERS = L3_ROUTER + 's'
 L3_AGENT = 'l3-agent'
@@ -42,10 +43,11 @@ L3_AGENTS = L3_AGENT + 's'
 
 class RouterSchedulerController(wsgi.Controller):
     def get_plugin(self):
-        plugin = directory.get_plugin(plugin_constants.L3)
+        plugin = manager.NeutronManager.get_service_plugins().get(
+            service_constants.L3_ROUTER_NAT)
         if not plugin:
-            LOG.error('No plugin for L3 routing registered to handle '
-                      'router scheduling')
+            LOG.error(_('No plugin for L3 routing registered to handle '
+                        'router scheduling'))
             msg = _('The resource could not be found.')
             raise webob.exc.HTTPNotFound(msg)
         return plugin
@@ -63,37 +65,34 @@ class RouterSchedulerController(wsgi.Controller):
         policy.enforce(request.context,
                        "create_%s" % L3_ROUTER,
                        {})
-        agent_id = kwargs['agent_id']
-        router_id = body['router_id']
-        result = plugin.add_router_to_l3_agent(request.context, agent_id,
-                                               router_id)
-        notify(request.context, 'l3_agent.router.add', router_id, agent_id)
-        return result
+        return plugin.add_router_to_l3_agent(
+            request.context,
+            kwargs['agent_id'],
+            body['router_id'])
 
     def delete(self, request, id, **kwargs):
         plugin = self.get_plugin()
         policy.enforce(request.context,
                        "delete_%s" % L3_ROUTER,
                        {})
-        agent_id = kwargs['agent_id']
-        result = plugin.remove_router_from_l3_agent(request.context, agent_id,
-                                                    id)
-        notify(request.context, 'l3_agent.router.remove', id, agent_id)
-        return result
+        return plugin.remove_router_from_l3_agent(
+            request.context, kwargs['agent_id'], id)
 
 
 class L3AgentsHostingRouterController(wsgi.Controller):
     def get_plugin(self):
-        plugin = directory.get_plugin(plugin_constants.L3)
+        plugin = manager.NeutronManager.get_service_plugins().get(
+            service_constants.L3_ROUTER_NAT)
         if not plugin:
-            LOG.error('No plugin for L3 routing registered to handle '
-                      'router scheduling')
+            LOG.error(_('No plugin for L3 routing registered to handle '
+                        'router scheduling'))
             msg = _('The resource could not be found.')
             raise webob.exc.HTTPNotFound(msg)
         return plugin
 
     def index(self, request, **kwargs):
-        plugin = self.get_plugin()
+        plugin = manager.NeutronManager.get_service_plugins().get(
+            service_constants.L3_ROUTER_NAT)
         policy.enforce(request.context,
                        "get_%s" % L3_AGENTS,
                        {})
@@ -101,7 +100,7 @@ class L3AgentsHostingRouterController(wsgi.Controller):
             request.context, kwargs['router_id'])
 
 
-class L3agentscheduler(api_extensions.ExtensionDescriptor):
+class L3agentscheduler(extensions.ExtensionDescriptor):
     """Extension class supporting l3 agent scheduler.
     """
 
@@ -117,6 +116,10 @@ class L3agentscheduler(api_extensions.ExtensionDescriptor):
     def get_description(cls):
         return "Schedule routers among l3 agents"
 
+    @classmethod
+    def get_namespace(cls):
+        return "http://docs.openstack.org/ext/l3_agent_scheduler/api/v1.0"
+
     @classmethod
     def get_updated(cls):
         return "2013-02-07T10:00:00-00:00"
@@ -129,7 +132,7 @@ class L3agentscheduler(api_extensions.ExtensionDescriptor):
                       collection_name="agents")
 
         controller = resource.Resource(RouterSchedulerController(),
-                                       faults.FAULT_MAP)
+                                       base.FAULT_MAP)
         exts.append(extensions.ResourceExtension(
             L3_ROUTERS, controller, parent))
 
@@ -137,7 +140,7 @@ class L3agentscheduler(api_extensions.ExtensionDescriptor):
                       collection_name="routers")
 
         controller = resource.Resource(L3AgentsHostingRouterController(),
-                                       faults.FAULT_MAP)
+                                       base.FAULT_MAP)
         exts.append(extensions.ResourceExtension(
             L3_AGENTS, controller, parent))
         return exts
@@ -146,72 +149,43 @@ class L3agentscheduler(api_extensions.ExtensionDescriptor):
         return {}
 
 
-class InvalidL3Agent(agent_exc.AgentNotFound):
+class InvalidL3Agent(agent.AgentNotFound):
     message = _("Agent %(id)s is not a L3 Agent or has been disabled")
 
 
 class RouterHostedByL3Agent(exceptions.Conflict):
-    message = _("The router %(router_id)s has been already hosted "
-                "by the L3 Agent %(agent_id)s.")
+    message = _("The router %(router_id)s has been already hosted"
+                " by the L3 Agent %(agent_id)s.")
 
 
 class RouterSchedulingFailed(exceptions.Conflict):
-    message = _("Failed scheduling router %(router_id)s to "
-                "the L3 Agent %(agent_id)s.")
+    message = _("Failed scheduling router %(router_id)s to"
+                " the L3 Agent %(agent_id)s.")
 
 
-class RouterReschedulingFailed(exceptions.Conflict):
-    message = _("Failed rescheduling router %(router_id)s: "
-                "no eligible l3 agent found.")
+class RouterNotHostedByL3Agent(exceptions.Conflict):
+    message = _("The router %(router_id)s is not hosted"
+                " by L3 agent %(agent_id)s.")
 
 
-class RouterL3AgentMismatch(exceptions.Conflict):
-    message = _("Cannot host distributed router %(router_id)s "
-                "on legacy L3 agent %(agent_id)s.")
-
-
-class DVRL3CannotAssignToDvrAgent(exceptions.Conflict):
-    message = _("Not allowed to manually assign a router to an "
-                "agent in 'dvr' mode.")
-
-
-class DVRL3CannotRemoveFromDvrAgent(exceptions.Conflict):
-    message = _("Not allowed to manually remove a router from "
-                "an agent in 'dvr' mode.")
-
-
-class RouterDoesntSupportScheduling(exceptions.Conflict):
-    message = _("Router %(router_id)s does not support agent scheduling.")
-
-
-class L3AgentSchedulerPluginBase(metaclass=abc.ABCMeta):
+class L3AgentSchedulerPluginBase(object):
     """REST API to operate the l3 agent scheduler.
 
     All of method must be in an admin context.
     """
 
-    @abc.abstractmethod
+    @abstractmethod
     def add_router_to_l3_agent(self, context, id, router_id):
         pass
 
-    @abc.abstractmethod
+    @abstractmethod
     def remove_router_from_l3_agent(self, context, id, router_id):
         pass
 
-    @abc.abstractmethod
+    @abstractmethod
     def list_routers_on_l3_agent(self, context, id):
         pass
 
-    @abc.abstractmethod
+    @abstractmethod
     def list_l3_agents_hosting_router(self, context, router_id):
         pass
-
-    def router_supports_scheduling(self, context, router_id):
-        """Override this method to conditionally schedule routers."""
-        return True
-
-
-def notify(context, action, router_id, agent_id):
-    info = {'id': agent_id, 'router_id': router_id}
-    notifier = n_rpc.get_notifier('router')
-    notifier.info(context, action, {'agent': info})
